﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using FTD2XX_NET;

namespace MNET_STK_AXIS_Sample
{
    public partial class Form1 : Form
    {
        ////////////////////////////////////////////////////////////////////
        /// <summary>
        ///     Version Information
        /// </summary>
        // public string Version = "1.00";
        // string VerDate = "2020.02.12";

        ////////////////////////////////////////////////////////////////////
        private byte[] Devinf_D;        // Keep Device Information value of 64 devices
        private uint[] Attr_D;          // Keep Attribute value of 64 devices
        private uint Status;            // Value of G9001A Status
        private uint IntStatus;         // Value of G9001A Interrupt Status
        private byte dno;               // Device Number

        private uint[] RegV_X;          // Keep G9103C Register value
        private uint[] RegV_SX;         // Save G9103C Register value
        private int[] Base_X;           // Base of each register 0: Hexadecimal 1: Decimal
        private int[] Keta_X;           // Hexadecimal digits of each register
        private int[] ReadF;            // 0: Register whose value does not change (reading unnecessary) 1: Register whose value changes (reading required)
        private int[] RegR;             // Register Read Command
        private Label[] Reg_X;          // Container for arranging labels that display register values
        private int MaxReg = 13;        // Maximum number of registers to display
        private int AllReadFlag = 0;    // Register read flag 0: Only change value 1: All value

        private int chkloop;            // Waiting loop
        private int CommFlag = 0;       // Timer processing flag
        private byte RCom = 0;          // Holds the register write command used for writing
        private uint WriteD;            // Data to be written to the register
        ////////////////////////////////////////////////////////////////////
        private clsFTDI hFTDI = new clsFTDI();
        private accessMNET hMNET = new accessMNET();
        FTDI.FT_STATUS ftStatus = FTDI.FT_STATUS.FT_OK;
        ////////////////////////////////////////////////////////////////////

        public Form1()
        {
            InitializeComponent();
            this.MaximizeBox = false;   // Not maximize
        }

        /// <summary>
        /// Software startup processing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_Load(object sender, EventArgs e)
        {
            //Arranged in the center of the screen
            this.Left = (System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width - this.Width) / 2;
            this.Top = (System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height - this.Height) / 2;
            this.StartPosition = FormStartPosition.Manual;

            Devinf_D = new byte[]  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,   // 00-15
                                     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,   // 16-31
                                     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,   // 32-47
                                     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }; // 48-63

            Attr_D = new uint[]   { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,    // 00-15
                                    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,    // 16-31
                                    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,    // 32-47
                                    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };  // 48-63

            RegV_X = new uint[] {0, 0,              // MSTSW, IOPW
                                 0, 0, 0, 0, 0, 0,  // PRMV, PRFL, PRFH, PRMG, PRMD, RCUN1
                                 0, 0, 0, 0, 0 };   // RSTS, REST, RIST, RPLS, RSPD

            RegV_SX = new uint[] {1, 1,             // MSTSW, IOPW
                                 1, 2, 1, 1, 1, 1,  // PRMV, PRFL, PRFH, PRMG, PRMD, RCUN1
                                 1, 1, 1, 1, 1 };   // RSTS, REST, RIST, RPLS, RSPD

            Base_X = new int[] {0, 0,               // MSTSW, IOPW
                                0, 0, 0, 0, 0, 0,   // PRMV, PRFL, PRFH, PRMG, PRMD, RCUN1
                                0, 0, 0, 0, 0 };    // RSTS, REST, RIST, RPLS, RSPD

            Keta_X = new int[] {4, 4,               // MSTSW, IOPW
                                7, 5, 5, 3, 8, 7,   // PRMV, PRFL, PRFH, PRMG, PRMD, RCUN1
                                8, 8, 8, 7, 5 };    // RSTS, REST, RIST, RPLS, RSPD

            ReadF = new int[] {1, 1,                // MSTSW, IOPW
                               1, 0, 0, 0, 0, 1,    // PRMV, PRFL, PRFH, PRMG, PRMD, RCUN1
                               1, 1, 1, 1, 1 };     // RSTS, REST, RIST, RPLS, RSPD

            RegR = new int[] { 0x00, 0x00,                         // MSTSW, IOPW
                               0xC0, 0xC1, 0xC2, 0xC5, 0xC7, 0xE3, // PRMV, PRFL, PRFH, PRMG, PRMD, RCUN1
                               0xF1, 0xF2, 0xF3, 0xF4, 0xF5 };     // RSTS, REST, RIST, RPLS, RSPD

            Reg_X = new Label[] {mstsw_lx, iopw_lx,                                          // MSTSW, IOPW
                                 prmv_lx , prfl_lx , prfh_lx , prmg_lx , prmd_lx , rcun1_lx, // PRMV, PRFL, PRFH, PRMG, PRMD, RCUN1
                                 rsts_lx , rest_lx , rist_lx , rpls_lx , rspd_lx };          // RSTS, REST, RIST, RPLS, RSPD

            this.p_sbsy.Image = this.imageList1.Images[0]; // BUSY LED OFF
            this.p_sdir.Image = this.imageList1.Images[0]; // DIR LED OFF
            this.p_serr.Image = this.imageList1.Images[0]; // ERROR INTERRUPT LED OFF
            this.p_sevt.Image = this.imageList1.Images[0]; // IVENT INTERRUPT LED OFF
            this.p_sfu.Image = this.imageList1.Images[0];  // During acceleration LED OFF
            this.p_sfd.Image = this.imageList1.Images[0];  // During deceleration LED OFF
            this.p_sfc.Image = this.imageList1.Images[0];  // During constant speed LED OFF
            this.p_salm.Image = this.imageList1.Images[0]; // ALARM LED OFF
            this.p_spel.Image = this.imageList1.Images[0]; // +EL LED OFF
            this.p_smel.Image = this.imageList1.Images[0]; // -EL LED OFF
            this.p_sorg.Image = this.imageList1.Images[0]; // ORG LED OFF
            this.p_ssd.Image = this.imageList1.Images[0];  // SD LED OFF

            ////////////////////////////////////////////////////////////////////
            ftStatus = hFTDI.LSI_Open("NPM G9001A-EV Paralell access A");

            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {
                MessageBox.Show(" G9001A not found" + Environment.NewLine + " Connect Motionnet Stater Kit!", "Error");
                Close();
            }

            ftStatus = hMNET.MNETCenterComdWriteSoftwareReset(); // Software Reset Command
            chkloop = 1; // Waiting loop set
            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {
                goto END;
            }
            else
            {
                while (chkloop != 0)
                {
                    ftStatus = hMNET.MNETCenterReadSts(ref Status, ref IntStatus); // Read Status
                    if (ftStatus != FTDI.FT_STATUS.FT_OK)
                    {
                        goto END;
                    }
                    if ((Status & 0x2000) == 0) // RBSY check
                    {
                        chkloop = 0; // Reset processing completed
                    }
                }
            }
            ftStatus = hMNET.MNETCenterComdWriteSystemComm(); // System communication Command
            chkloop = 1; // Waiting loop set
            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {
                goto END;
            }
            else
            {
                while (chkloop != 0)
                {
                    ftStatus = hMNET.MNETCenterReadSts(ref Status, ref IntStatus); // Read Status
                    if (ftStatus != FTDI.FT_STATUS.FT_OK)
                    {
                        goto END;
                    }
                    if ((Status & 0x0001) == 1) // CEND check
                    {
                        chkloop = 0; // Reset processing completed
                    }
                }
            }
            ftStatus = hMNET.MNETCenterComdWriteCyclicComm(); // Start cyclic communication Command
            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {
                goto END;
            }

            // G9103C(Axis) Search
            Device_Search();
            // G9103C Inital data setting
            Axis_InitSet();
            // G9103C Status & Register data Read
            AllReadFlag = 1;
            Axis_StatusRead();
            Axis_RegisterRead();
            // All Data Display
            AllDataDisp();
            AllReadFlag = 0;
            this.timer1.Start();

END:
            FTDI_processing();
        }

        /// <summary>
        /// Software termination processing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            hFTDI.LSI_Close();
        }

        /// <summary>
        /// FTDI processing
        /// </summary>
        private void FTDI_processing()
        {
            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {
                this.timer1.Stop();
                MessageBox.Show(" FTDI Commucation Error" + Environment.NewLine + " Check USB connector connection!", "Error");
                Close();
            }
        }

        /// <summary>
        /// Device Search [G9103C(Axis)]
        /// </summary>
        private void Device_Search()
        {
            int loops;
            int chk = 0;

            /// Address Map Device Information All Read
            for (loops = 0; loops < 64; loops++)
            {
                dno = (byte)loops;
                ftStatus = hMNET.MNETCenterReadInf(dno, ref Devinf_D);
                if (ftStatus != FTDI.FT_STATUS.FT_OK)
                {
                    goto END;
                }
            }

            /// Device Information Command All Read
            //            byte rdataA = 0, rdataB = 0;
            //            for (loops = 0; loops < 32; loops++)
            //            {
            //                dno = (byte)(loops * 2);
            //                ftStatus = hMNET.MNETCenterComdReadDevInf(dno, ref rdataA, ref rdataB);
            //                if (ftStatus != FTDI.FT_STATUS.FT_OK)
            //                {
            //                    goto END;
            //                }
            //                else
            //                {
            //                    Devinf_D[dno + 0] = rdataA;
            //                    Devinf_D[dno + 1] = rdataB;
            //                }
            //            }

            for (loops = 0; loops < 64; loops++)
            {
                dno = (byte)loops;
                if (Devinf_D[dno] == 0x8B) // G9103C(Axis) Device Information Check
                {
                    ftStatus = hMNET.MNETCenterComdReadDevAttribute(dno); // Device Attribute Read
                    if (ftStatus != FTDI.FT_STATUS.FT_OK)
                    {
                        goto END;
                    }
                    while (chkloop != 0)
                    {
                        ftStatus = hMNET.MNETCenterReadSts(ref Status, ref IntStatus);
                        if (ftStatus != FTDI.FT_STATUS.FT_OK)
                        {
                            goto END;
                        }
                        if ((Status & 0x0001) == 1) // CEND
                        {
                            chkloop = 0; // Reset processing completed
                        }
                    }
                    ftStatus = hMNET.MNETCenterComdGetDevAttribute(dno, ref Attr_D); // Device Attribute Data Get
                    if (ftStatus != FTDI.FT_STATUS.FT_OK)
                    {
                        goto END;
                    }

                    if ((Attr_D[dno] & 0x00238100) == 0x00238100) // G9103C Device Attribute Information Check
                    {
                        t_dno.Text = string.Format("{0:D" + 2 + "}", dno);
                        t_dinf.Text = string.Format("{0:X" + 2 + "}", Devinf_D[dno]);
                        t_dattr.Text = string.Format("{0:X" + 8 + "}", Attr_D[dno]);
                        chk = 1;
                        break;
                    }
                }
            }
            if (chk == 0)
            {
                this.timer1.Stop();
                MessageBox.Show(" G9103C(Axis) not found" + Environment.NewLine + " Connect [ G9002A_G9103C-EV ]!", "Error");
                Close();
            }

END:
            FTDI_processing();
        }

        /// <summary>
        /// Axis initial setting
        /// </summary>
        private void Axis_InitSet()
        {
            ftStatus = hMNET.MNETLocalRegisterWrite(dno, 0x9C, 0x00000002); // write RENV1(PMD = 0x02)
            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {
                goto END;
            }

            ftStatus = hMNET.MNETLocalRegisterWrite(dno, 0x9D, 0x000000FF); // write RENV2(P0M-P7M = 0xFF)
            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {
                goto END;
            }

            ftStatus = hMNET.MNETLocalRegisterWrite(dno, 0x9F, 0x81000000); // write RENV4(LTM = 0x01, ISMR = 1)
            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {
                goto END;
            }

            ftStatus = hMNET.MNETLocalRegisterWrite(dno, 0xAC, 0x00000C01); // write RIRQ(IREN = 1, IROL = 1, IRSD = 1)
            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {
                goto END;
            }

            ftStatus = hMNET.MNETLocalRegisterWrite(dno, 0xB1, 0x00000001); // write PRFL = 1pps
            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {
                goto END;
            }

            ftStatus = hMNET.MNETLocalRegisterWrite(dno, 0xB2, 0x000005DC); // write PRFH = 1500pps
            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {
                goto END;
            }

            ftStatus = hMNET.MNETLocalRegisterWrite(dno, 0xB3, 0x00000D07); // write PRUR = 3335(1sec)
            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {
                goto END;
            }

            ftStatus = hMNET.MNETLocalRegisterWrite(dno, 0xB5, 0x000000C7); // write PRMG = 199 (Speed magnification = 1)
            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {
                goto END;
            }

            ftStatus = hMNET.MNETLocalRegisterWrite(dno, 0xB7, 0x00000041); // write PRMD(MOD = 0x41: Positioning operation)

END:
            FTDI_processing();
        }

        /// <summary>
        /// Axis Device status read
        /// </summary>
        private void Axis_StatusRead()
        {
            byte data_a = 0;
            byte data_b = 0;

            ftStatus = hMNET.MNETCenterComdReadPortD(dno, 0, ref data_a, ref data_b);
            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {
                goto END;
            }
            else
            {
                RegV_X[0] = (uint)(data_b * 256 + data_a); // STBW
            }

            ftStatus = hMNET.MNETCenterComdReadPortD(dno, 1, ref data_a, ref data_b);
            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {
                goto END;
            }
            else
            {
                RegV_X[1] = (uint)(data_b * 256 + data_a); // IOPW
            }

        END:
            FTDI_processing();
        }

        /// <summary>
        /// Axis register read
        /// </summary>
        private void Axis_RegisterRead()
        {
            int Min = 2;
            int Max = MaxReg;

            ftStatus = hMNET.MNETLocalAllRegisterRead(dno, ref RegR, ref ReadF, Min, Max, AllReadFlag);
            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {
                goto END;
            }
            ftStatus = hMNET.MNETLocalAllRegisterGet(ref RegR, ref ReadF, Min, Max, AllReadFlag, ref RegV_X);

END:
            FTDI_processing();
        }

        /// <summary>
        /// Axis register data display
        /// </summary>
        private void AllDataDisp()
        {
            int loops;

            if (ftStatus == FTDI.FT_STATUS.FT_OK)
            {
                for (loops = 0; loops < MaxReg; loops++)
                {
                    if ((RegV_X[loops] != RegV_SX[loops]))
                    {
                        DispReg(loops);
                        RegV_SX[loops] = RegV_X[loops];
                        DispLed();
                    }
                }
            }
        }

        //////////////////////////////////////////////////////////////////
        // Display register values
        // idx: Tag number of label
        private void DispReg(int idx)
        {
            String str;

            if ((idx == 2) || (idx == 7)) // PRMV,RCUN1
            {
                str = Int2StrS(RegV_X[idx], Keta_X[idx], Base_X[idx], 1);
            }
            else
            {
                str = Int2Str(RegV_X[idx], Keta_X[idx], Base_X[idx], 1);
            }
            Reg_X[idx].Text = str;

            if (RegV_X[idx] == 0)
            {
                Reg_X[idx].ForeColor = Color.Black;
            }
            else
            {
                Reg_X[idx].ForeColor = Color.Red;
            }
        }

        /// <summary>
        /// Display LED
        private void DispLed()
        {
            SetLed(this.p_sbsy, RegV_X[0], 0x0100, 1); // SBSY
            SetLed(this.p_serr, RegV_X[0], 0x0004, 2); // SERR
            SetLed(this.p_sevt, RegV_X[0], 0x0008, 1); // SEVT

            SetLed(this.p_sdir, RegV_X[8], 0x00000010, 1); // SDIR
            SetLed(this.p_salm, RegV_X[8], 0x00000020, 2); // SALM
            SetLed(this.p_spel, RegV_X[8], 0x00000040, 2); // SPEL
            SetLed(this.p_smel, RegV_X[8], 0x00000080, 2); // SMEL
            SetLed(this.p_sorg, RegV_X[8], 0x00000100, 1); // SORG
            SetLed(this.p_ssd, RegV_X[8], 0x00000200, 1);  // SSD

            switch (RegV_X[8] & 0x0000000F)
            {
                case 0x00000006: // Feeding at FA constant speed
                case 0x00000007: // Feeding at FL constant speed
                case 0x00000009: // Feeding at FH constant speed
                    this.p_sfu.Image = this.imageList1.Images[0];
                    this.p_sfd.Image = this.imageList1.Images[0];
                    this.p_sfc.Image = this.imageList1.Images[1];
                    break;

                case 0x00000008: // Accelerating
                    this.p_sfu.Image = this.imageList1.Images[1];
                    this.p_sfd.Image = this.imageList1.Images[0];
                    this.p_sfc.Image = this.imageList1.Images[0];
                    break;

                case 0x0000000A: // Decelerating
                    this.p_sfu.Image = this.imageList1.Images[0];
                    this.p_sfd.Image = this.imageList1.Images[1];
                    this.p_sfc.Image = this.imageList1.Images[0];
                    break;

                default:
                    this.p_sfu.Image = this.imageList1.Images[0];
                    this.p_sfd.Image = this.imageList1.Images[0];
                    this.p_sfc.Image = this.imageList1.Images[0];
                    break;
            }
        }
     
        /// <summary>
        /// Display set of status
        /// </summary>
        /// <param name="pb"></param>
        /// <param name="status"></param>
        /// <param name="mask"></param>
        /// <param name="type"></param>
        private void SetLed(PictureBox pb, uint status, int mask, int type)
        {
            if ((status & mask) == 0)
            {
                pb.Image = this.imageList1.Images[0];     // OFF
            }
            else
            {
                pb.Image = this.imageList1.Images[type];  // ON
            }
        }

        //////////////////////////////////////////////////////////////////
        // Convert number to string
        //  kt：Specify the number of digits in hexadecimal
        //  bs：0 = Hexadecimal,  1 = Decimal number
        //  mk: 0 = Do not add anything, 1 = Add [ h]
        public String Int2Str(uint wr1, int kt, int bs, int mk)
        {
            String keta;

            if (bs == 1) // Decimal number
            {
                return (string.Format("{0:D}", wr1));
            }
            else // Hexadecimal
            {
                keta = "{0:X" + kt + "}";
                if (mk == 1)
                {
                    return (string.Format(keta, wr1) + " h");
                }
                else
                {
                    return (string.Format(keta, wr1));
                }
            }
        }

        //////////////////////////////////////////////////////////////////
        // Convert number to string
        //  kt：Specify the number of digits in hexadecimal
        //  bs：0 = Hexadecimal,  1 = Decimal number (signed with 8 bytes)
        //  mk: 0 = Do not add anything, 1 = Add [ h]
        public String Int2StrS(uint wr1, int kt, int bs, int mk)
        {
            String keta;

            if (bs == 1) // Decimal number
            {
                return (string.Format("{0:D}", (int)wr1));
            }
            else // Hexadecimal
            {
                keta = "{0:X" + kt + "}";
                if (mk == 1)
                {
                    return (string.Format(keta, wr1) + " h");
                }
                else
                {
                    return (string.Format(keta, wr1));
                }
            }
        }

        /// <summary>
        /// Right-click on the register display
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void reg_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                String str = (String)((Label)sender).Tag;
                int idx = int.Parse(str);

                if (Base_X[idx] == 0) // Decimal number
                {
                    Base_X[idx] = 1;
                }
                else // Hexadecimal
                {
                    Base_X[idx] = 0;
                }
                DispReg(idx);
            }
        }

        /// <summary>
        /// CCW button processing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_ccw_Click(object sender, EventArgs e)
        {
            if ((RegV_X[0] & 0x0100) == 0) // MSTSW.SBSY Check
            {
                ftStatus = hMNET.MNETLocalRegisterWrite(dno, 0xB0, 0xFFFFEE00); // write PRMV = -4608
                FTDI_processing();

                RCom = (byte)0x53; // High-speed start
                CommFlag = 1;
            }
        }

        /// <summary>
        /// CW button processing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_cw_Click(object sender, EventArgs e)
        {
            if ((RegV_X[0] & 0x0100) == 0) // MSTSW.SBSY Check
            {
                ftStatus = hMNET.MNETLocalRegisterWrite(dno, 0xB0, 0x00001200); // write PRMV = 4608
                FTDI_processing();

                RCom = (byte)0x53; // High-speed start
                CommFlag = 1;
            }
        }

        /// <summary>
        /// STOP button processing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_stop_Click(object sender, EventArgs e)
        {
            RCom = (byte)0x4A; // Decelerate and stop
            CommFlag = 1;
        }

        /// <summary>
        /// RCUN1 CLEAR button processing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_rcun1_clr_Click(object sender, EventArgs e)
        {
            RCom = (byte)0x20; // Reset COUNTER1 (command position)
            CommFlag = 1;
        }

        /// <summary>
        /// REST CLEAR button processing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_rest_clr_Click(object sender, EventArgs e)
        {
            WriteD = RegV_X[9];
            RCom = (byte)0xAD; // Write REST register
            CommFlag = 2;
        }

        /// <summary>
        /// RIST CLEAR button processing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_rist_clr_Click(object sender, EventArgs e)
        {
            WriteD = RegV_X[10];
            RCom = (byte)0xAE; // Write RIST register
            CommFlag = 2;
        }

        /// <summary>
        /// Device Error Check [G9103C(Axis)]
        /// </summary>
        private void Device_Error_Check()
        {
            ftStatus = hMNET.MNETCenterReadSts(ref Status, ref IntStatus); // Read Status
            if ((Status & 0x0008) != 0) // EIOE
            {
                this.timer1.Stop();
                MessageBox.Show(" G9103C(Axis) not found" + Environment.NewLine + " Connect [ G9002A_G9103C-EV ]!", "Error");
                Close();
            }
            FTDI_processing();
        }

        /// <summary>
        /// Timer processing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1_Tick(object sender, EventArgs e)
        {
            switch (CommFlag)
            {
                case 0:
                    break;

                case 1:
                    ftStatus = hMNET.MNETLocalWrite_COM(dno, RCom);
                    if (ftStatus != FTDI.FT_STATUS.FT_OK)
                    {
                        goto END;
                    }
                    CommFlag = 0;
                    break;

                case 2:
                    ftStatus = hMNET.MNETLocalRegisterWrite(dno, (int)RCom, (uint)WriteD);
                    if (ftStatus != FTDI.FT_STATUS.FT_OK)
                    {
                        goto END;
                    }
                    CommFlag = 0;
                    break;
            }
            Axis_StatusRead();
            Axis_RegisterRead();
            AllDataDisp();
            Device_Error_Check();

END:
            FTDI_processing();
        }
    }
}
